"use strict";
function _export(target, all) {
    for(var name in all)Object.defineProperty(target, name, {
        enumerable: true,
        get: all[name]
    });
}
_export(exports, {
    basename: ()=>basename,
    Kondo: ()=>Kondo
});
const _githubActionsCoreTypedInputs = require("@broadshield/github-actions-core-typed-inputs");
const _githubActionsOctokitHydrated = require("@broadshield/github-actions-octokit-hydrated");
function basename(path) {
    if (!path) return '';
    return path.split('/').reverse()[0] ?? '';
}
const releasesRoute = 'GET /repos/{owner}/{repo}/releases';
const tagsRoute = 'GET /repos/{owner}/{repo}/tags';
const workflowsRoute = 'GET /repos/{owner}/{repo}/actions/workflows';
const workflowRunsRoute = 'GET /repos/{owner}/{repo}/actions/workflows/{workflow_id}/runs';
const availableRoutes = {
    [releasesRoute]: {
        returnKey: 'id',
        lookupKey: 'release_id'
    },
    [tagsRoute]: {
        returnKey: 'ref',
        lookupKey: 'ref'
    },
    [workflowsRoute]: {
        returnKey: 'id',
        lookupKey: 'workflow_id'
    },
    [workflowRunsRoute]: {
        returnKey: 'id',
        lookupKey: 'run_id'
    }
};
function isKeyOfResponseData(arg, responseData) {
    return typeof arg === 'string' && Object.keys(responseData).includes(arg) && typeof responseData[arg] in [
        'string',
        'number'
    ];
}
function parametersToString(parameters) {
    if (!parameters) return '';
    return Object.entries(parameters).map(([key, value])=>`${key}=${value}`).join('&');
}
function itemFilterToString(filter) {
    if (filter) {
        const keys = Object.keys(filter);
        const outputStringArray = [];
        for (const key of keys){
            if (isKeyOfResponseData(key, filter)) {
                const value = filter[key];
                let valueString = '';
                if (value instanceof RegExp) {
                    valueString = `RegEx of ${value.toString()}`;
                } else if (value instanceof Date) {
                    valueString = value.toISOString();
                } else if (typeof value === 'string') {
                    valueString = value;
                } else if (typeof value === 'number') {
                    valueString = Number(value).toString(10);
                }
                if (valueString) {
                    outputStringArray.push(`[${String(key)}: ${valueString}]`);
                }
            }
        }
        if (outputStringArray.length > 0) {
            return outputStringArray.join(', ');
        }
    }
    return 'undefined';
}
function repoToString(repo) {
    return `${repo.owner}/${repo.repo}`;
}
function assertOctokit(octokit) {
    if (!octokit) {
        throw new Error('Octokit instance not set');
    }
}
let Kondo = class Kondo {
    setOctokit(octokit) {
        this.octokit = octokit;
    }
    getOctokit() {
        return this.octokit;
    }
    async getMatchingItemsFromEndpointRest(endpoint, parameters, filter, octokit = this.octokit) {
        assertOctokit(octokit);
        const filtered_return_keys = [];
        _githubActionsCoreTypedInputs.logger.debug(`getMatchingItemsFromEndpointRest: endpoint: ${endpoint}, parameters: ${parametersToString(parameters)}, filter: ${itemFilterToString(filter)}`);
        for await (const response of octokit.paginate.iterator(endpoint, parameters)){
            // ResponseType<T>["data"]
            if (response?.data && Array.isArray(response.data)) {
                const items = response.data;
                for (const item of items){
                    const rKey = availableRoutes[endpoint].returnKey;
                    if (item[rKey] !== undefined) {
                        const itemReturnKey = item[rKey];
                        if (!filter) {
                            filtered_return_keys.push(itemReturnKey);
                        } else {
                            const filterKeys = Object.keys(filter);
                            for (const filterKey of filterKeys){
                                if (filterKey && isKeyOfResponseData(filterKey, item)) {
                                    const filterValue = filter[filterKey];
                                    if (filterValue instanceof RegExp) {
                                        if (filterValue.test(item[filterKey])) {
                                            filtered_return_keys.push(itemReturnKey);
                                        }
                                    } else if (filterValue instanceof Date) {
                                        if (filterValue.getTime() === new Date(item[filterKey]).getTime()) {
                                            filtered_return_keys.push(itemReturnKey);
                                        }
                                    } else if (typeof filterValue === 'string' || typeof filterValue === 'number') {
                                        if (filterValue === item[filterKey]) {
                                            filtered_return_keys.push(itemReturnKey);
                                        }
                                    } else {
                                        _githubActionsCoreTypedInputs.logger.warn(`Unknown filter value type: ${filterValue} as ${typeof filterValue}`);
                                    }
                                } else {
                                    _githubActionsCoreTypedInputs.logger.warn(`Unknown filter key: ${String(filterKey)} as ${typeof filterKey}`);
                                }
                            }
                        }
                    }
                }
            }
        }
        _githubActionsCoreTypedInputs.logger.info(`Found ${filtered_return_keys.length} ids matching filter: ${itemFilterToString(filter)}`);
        return filtered_return_keys;
    }
    async getFilteredReleaseIdsFromRepo(parameters, filter, repo = this.repo) {
        return this.getMatchingItemsFromEndpointRest(releasesRoute, {
            ...repo,
            ...parameters
        }, filter);
    }
    async getFilteredTagRefsFromRepo(parameters, filter, repo = this.repo) {
        return this.getMatchingItemsFromEndpointRest(tagsRoute, {
            ...repo,
            ...parameters
        }, filter);
    }
    async getFilteredWorkflowIdsFromRepo(parameters, filter, repo = this.repo) {
        return this.getMatchingItemsFromEndpointRest(workflowsRoute, {
            ...repo,
            ...parameters
        }, filter);
    }
    async getFilteredWorkflowRunIdsFromRepo(parameters, filter, repo = this.repo) {
        return this.getMatchingItemsFromEndpointRest(workflowRunsRoute, {
            ...repo,
            ...parameters
        }, filter);
    }
    async deleteReleasesByIds(release_ids, repo = this.repo, octokit = this.octokit) {
        assertOctokit(octokit);
        return Promise.all(release_ids.map(async (release_id)=>octokit.rest.repos.deleteRelease({
                ...repo,
                release_id
            }).then((value)=>_githubActionsCoreTypedInputs.logger.debug(`Successfully deleted release ${release_id} in repo ${repoToString(repo)}. status: ${value.status}`), (error)=>{
                _githubActionsCoreTypedInputs.logger.error(`Error deleting release ${release_id} in repo ${repoToString(repo)}`);
                if (error instanceof Error || typeof error === 'string') {
                    _githubActionsCoreTypedInputs.logger.error(error);
                } else {
                    _githubActionsCoreTypedInputs.logger.error(JSON.stringify(error));
                }
            })));
    }
    async deleteTagsByRefs(refs, repo = this.repo, octokit = this.octokit) {
        assertOctokit(octokit);
        return Promise.all(refs.map(async (ref)=>octokit.rest.git.deleteRef({
                ...repo,
                ref
            }).then((value)=>_githubActionsCoreTypedInputs.logger.debug(`Successfully deleted tag ${ref} in repo ${repoToString(repo)}. status: ${value.status}`), (error)=>{
                _githubActionsCoreTypedInputs.logger.error(`Error deleting tag ${ref} in repo ${repoToString(repo)}`);
                if (error instanceof Error || typeof error === 'string') {
                    _githubActionsCoreTypedInputs.logger.error(error);
                } else {
                    _githubActionsCoreTypedInputs.logger.error(JSON.stringify(error));
                }
            })));
    }
    /**
   *
   * @param {number[]} run_ids List of workflow run ids to cancel
   * @param {GithubOctokitRepo} repo Object containing repository's owner and repo keys, defaults to this.repo
   * @param {OctokitInstance} octokit Octokit instance to use, defaults to this.octokit
   * @returns {Promise<void[]>} Array of promises that resolve to void
   */ async deleteWorkflowRunsByIds(run_ids, repo = this.repo, octokit = this.octokit) {
        assertOctokit(octokit);
        return Promise.all(run_ids.map(async (run_id)=>octokit.rest.actions.deleteWorkflowRun({
                ...repo,
                run_id
            }).then((value)=>_githubActionsCoreTypedInputs.logger.debug(`Successfully deleted workflow run ${run_id} in repo ${repoToString(repo)}. status: ${value.status}`), (error)=>{
                _githubActionsCoreTypedInputs.logger.error(`Error deleting workflow run ${run_id} in repo ${repoToString(repo)}`);
                if (error instanceof Error || typeof error === 'string') {
                    _githubActionsCoreTypedInputs.logger.error(error);
                } else {
                    _githubActionsCoreTypedInputs.logger.error(JSON.stringify(error));
                }
            })));
    }
    constructor(options){
        this.repo = options.repo;
        if (options.github_token && !options.octokit) {
            this.octokit = (0, _githubActionsOctokitHydrated.createOctokit)(options.github_token, options.octokit);
        } else if (options.octokit) {
            this.octokit = options.octokit;
        }
    }
};

//# sourceMappingURL=github-actions-workflow-marie-kondo.js.map